# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..commons.errors.server_error import ServerError
from ..commons.errors.unauthorized_error import UnauthorizedError
from ..commons.types.error_body import ErrorBody
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from .types.list_outbound_response import ListOutboundResponse
from .types.outbound_purchase import OutboundPurchase
from .types.outbound_purchase_response import OutboundPurchaseResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OutboundNumbersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def purchase(
        self, *, request: OutboundPurchase, request_options: typing.Optional[RequestOptions] = None
    ) -> OutboundPurchaseResponse:
        """
        Purchase and configure a new outbound phone number. ($15/mo. subscription using your stored payment method).

        Parameters:
            - request: OutboundPurchase.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/outbound/purchase"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OutboundPurchaseResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListOutboundResponse:
        """
        Retrieves a list of all outbound phone numbers configured for your account, along with their associated settings.

        Parameters:
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/outbound"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListOutboundResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncOutboundNumbersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def purchase(
        self, *, request: OutboundPurchase, request_options: typing.Optional[RequestOptions] = None
    ) -> OutboundPurchaseResponse:
        """
        Purchase and configure a new outbound phone number. ($15/mo. subscription using your stored payment method).

        Parameters:
            - request: OutboundPurchase.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/outbound/purchase"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OutboundPurchaseResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListOutboundResponse:
        """
        Retrieves a list of all outbound phone numbers configured for your account, along with their associated settings.

        Parameters:
            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/outbound"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListOutboundResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .a_2_p_status_enum import A2PStatusEnum
from .check_status_request import CheckStatusRequest
from .check_status_response import CheckStatusResponse
from .get_messages_request import GetMessagesRequest
from .get_messages_response import GetMessagesResponse
from .legal_structure_enum import LegalStructureEnum
from .register_request import RegisterRequest
from .register_response import RegisterResponse
from .toggle_reply_enum import ToggleReplyEnum
from .toggle_reply_request import ToggleReplyRequest
from .toggle_reply_response import ToggleReplyResponse
from .trusted_user import TrustedUser
from .update_prompt_request import UpdatePromptRequest
from .update_prompt_response import UpdatePromptResponse
from .update_webhook_request import UpdateWebhookRequest
from .update_webhook_response import UpdateWebhookResponse
from .vertical_enum import VerticalEnum

__all__ = [
    "A2PStatusEnum",
    "CheckStatusRequest",
    "CheckStatusResponse",
    "GetMessagesRequest",
    "GetMessagesResponse",
    "LegalStructureEnum",
    "RegisterRequest",
    "RegisterResponse",
    "ToggleReplyEnum",
    "ToggleReplyRequest",
    "ToggleReplyResponse",
    "TrustedUser",
    "UpdatePromptRequest",
    "UpdatePromptResponse",
    "UpdateWebhookRequest",
    "UpdateWebhookResponse",
    "VerticalEnum",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class CustomTool(pydantic.BaseModel):
    name: str = pydantic.Field()
    """
    The name of the tool.
    """

    description: str = pydantic.Field()
    """
    A short explanation of what the tool does.
    """

    input_schema: typing.Any = pydantic.Field()
    """
    A JSON schema describing the input data.
    """

    speech: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string that will be spoken to the agent while your tool waits for a response.
    """

    response_data: typing.List[typing.Any] = pydantic.Field()
    """
    An array of objects that describe how to extract data from the response. Within the response data, you can create variables that the phone agent can reference in its prompt.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from .method_enum import MethodEnum
from .response_data import ResponseData

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class DynamicData(pydantic.BaseModel):
    """
    from blandai import DynamicData, ResponseData

    DynamicData(
        url="https://api.coindesk.com/v1/bpi/currentprice.json",
        response_data=[
            ResponseData(
                name="BTC Price USD",
                data="bpi.USD.rate",
                context="Current BTC Price: $BTC Price USD USD",
            ),
            ResponseData(
                name="BTC Price EUR",
                data="bpi.EUR.rate",
                context="In Euros: BTC Price USD EUR",
            ),
        ],
    )
    """

    url: str = pydantic.Field()
    """
    The URL of the external API to fetch data from.
    """

    response_data: typing.List[ResponseData] = pydantic.Field()
    """
    An array of objects describing how to parse and use the data fetched from the API.
    """

    method: typing.Optional[MethodEnum] = pydantic.Field(default=None)
    """
    Default value is `GET`.
    """

    cache: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to fetch the data once at the beginning of the call, or to re-check continuously for data that might change mid-call. Default value is `true`.
    """

    headers: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    An object of headers to send with the request.
    """

    body: typing.Optional[str] = pydantic.Field(default=None)
    """
    The body of the request.
    
    The following variables can be injected into the dynamic request body with double curly braces:
    
    - `from` (Ex. `+12223334444`)
    - `to`
    - `short_from` (Ex. `2223334444`)
    - `short_to`
    - `call_id`
    
    These string values will be replaced in each `dynamic_data[].body` where they're used by system values in each request.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing

VerticalEnum = typing.Literal[
    "AUTOMOTIVE",
    "AGRICULTURE",
    "BANKING",
    "CONSTRUCTION",
    "CONSUMER",
    "EDUCATION",
    "ENGINEERING",
    "ENERGY",
    "OIL_AND_GAS",
    "FAST_MOVING_CONSUMER_GOODS",
    "FINANCIAL",
    "FINTECH",
    "FOOD_AND_BEVERAGE",
    "GOVERNMENT",
    "HEALTHCARE",
    "HOSPITALITY",
    "INSURANCE",
    "LEGAL",
    "MANUFACTURING",
    "MEDIA",
    "ONLINE",
    "PROFESSIONAL_SERVICES",
    "RAW_MATERIALS",
    "REAL_ESTATE",
    "RELIGION",
    "RETAIL",
    "JEWELRY",
    "TECHNOLOGY",
    "TELECOMMUNICATIONS",
    "TRANSPORTATION",
    "TRAVEL",
    "ELECTRONICS",
    "NOT_FOR_PROFIT",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AnalysisObject,
    AnalyzeBatchRequest,
    BatchAnalysisObject,
    BatchCallData,
    BatchDetailsResponse,
    BatchObject,
    BatchParams,
    CallData,
    CallLength,
    CallLengthSummary,
    CallParams,
    ListBatchesResponse,
    QueueStatus,
    RetrieveBatchAnalysisResponse,
    SendBatchRequest,
    SendBatchResponse,
    StatusEnum,
    StopActiveBatchResponse,
)

__all__ = [
    "AnalysisObject",
    "AnalyzeBatchRequest",
    "BatchAnalysisObject",
    "BatchCallData",
    "BatchDetailsResponse",
    "BatchObject",
    "BatchParams",
    "CallData",
    "CallLength",
    "CallLengthSummary",
    "CallParams",
    "ListBatchesResponse",
    "QueueStatus",
    "RetrieveBatchAnalysisResponse",
    "SendBatchRequest",
    "SendBatchResponse",
    "StatusEnum",
    "StopActiveBatchResponse",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...calls.types.answered_by_enum import AnsweredByEnum
from ...core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class CallData(pydantic.BaseModel):
    created_at: dt.datetime = pydantic.Field()
    """
    The timestamp when the individual call was created.
    """

    to: str = pydantic.Field()
    """
    The phone number the call was made to.
    """

    from_: str = pydantic.Field(alias="from")
    """
    The phone number the call was made from.
    """

    completed: bool = pydantic.Field()
    """
    Indicates if the call was completed.
    """

    call_id: str = pydantic.Field()
    """
    The unique identifier for each individual call.
    """

    call_length: int = pydantic.Field()
    """
    The duration of the call in minutes.
    """

    answered_by: typing.Optional[AnsweredByEnum] = pydantic.Field(default=None)
    """
    Contains a string value if the batch had `answered_by_enabled` set to `true`.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..commons.types.phone_number import PhoneNumber
from ..commons.types.tools import Tools
from ..commons.types.voice_id import VoiceId
from ..commons.types.webhook import Webhook
from ..core.datetime_utils import serialize_datetime
from .dynamic_data import DynamicData
from .interruption_threshold import InterruptionThreshold
from .model_enum import ModelEnum
from .prounciation_guide import ProunciationGuide
from .temperature import Temperature
from .transfer_list import TransferList
from .voicemail_action import VoicemailAction

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class SendCall(pydantic.BaseModel):
    """
    from blandai import PronunciationObject, SendCall

    SendCall(
        phone_number="29382721828",
        task="Would love for you to check out our AI API!",
        temperature=0.5,
        transfer_list={
            "default": "+1234567890",
            "sales": "+1234567890",
            "support": "+1234567890",
            "billing": "+1234567890",
        },
        model="enhanced",
        pronunciation_guide=[
            PronunciationObject(
                word="API",
                pronunciation="A P I",
            ),
            PronunciationObject(
                word="AI",
                pronunciation="A I",
            ),
        ],
    )
    """

    phone_number: PhoneNumber = pydantic.Field()
    """
    The phone number to call. Country code defaults to `+1` (US) if not specified.
    
    Formatting is flexible, however for the most predictable results use the **[E.164](https://www.twilio.com/docs/glossary/what-e164#examples-of-e164-numbers)** format.
    """

    task: str = pydantic.Field()
    """
    Provide instructions, relevant information, and examples of the ideal conversation flow.
    """

    transfer_list: typing.Optional[TransferList] = pydantic.Field(default=None)
    """
    Give your agent the ability to transfer calls to a set of phone numbers.
    
    Overrides `transfer_phone_number` if a `transfer_list.default` is specified.
    
    Will default to `transfer_list.default`, or the chosen phone number.
    """

    model: typing.Optional[ModelEnum] = pydantic.Field(default=None)
    """
    Select a model to use for your call.
    
    In nearly all cases, `enhanced` is the best choice for now.
    """

    pronunciation_guide: typing.Optional[ProunciationGuide] = pydantic.Field(default=None)
    """
    The pronunciation guide is an `array` of `objects` that guides the LLM on how to say specific words. This is great for situations with complicated terms or names.
    """

    temperature: Temperature = pydantic.Field()
    """
    A value between 0 and 1 that controls the randomness of the LLM. 0 will cause more deterministic outputs while 1 will cause more random.
    """

    tools: Tools
    transfer_phone_number: typing.Optional[PhoneNumber] = pydantic.Field(default=None)
    """
    A phone number that the agent can transfer to under specific conditions - such as being asked to speak to a human or supervisor.
    """

    answered_by_enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables machine detection when the call starts to determine whether the call was answered by a person or a voicemail.
    
    Best Practices (when enabled) -
    
        - Since the determination is made at the beginning of the call, use `wait_for_greeting` to try and coax a human response.
    
        - If combined with first_sentence, try wording it so the person answering says something back - ex. `"Hello?"`
    
    Price - `$0.02` per call, however there is no charge for unanswered calls or calls that failed to send.
    """

    from_: typing.Optional[str] = pydantic.Field(alias="from", default=None)
    """
    Specify a purchased Outbound Number to call from. Country code is required, spaces or parentheses must be excluded.
    
    By default, calls are initiated from a separate pool of numbers owned by Bland.
    """

    reduce_latency: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Reducing latency means that the agent will generate responses more quickly and have less of a delay. This must be set to `true` when using Voice Clones.
    """

    voice_id: typing.Optional[VoiceId] = pydantic.Field(default=None)
    """
    Determines the voice of the AI agent, in conjunction with `reduce_latency`.
    
    Use the `GET /v1/voices endpoint` to see a full list of your available voices.
    
    To create your own Voice Clone, visit our [Custom Voice Cloning page](https://app.bland.ai/home?page=voice-clone).
    """

    voice_preset_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Use a voice preset instead of specifying individual voice settings.
    
    To create a voice preset, see [Create a Voice Preset](https://docs.bland.ai/api-v1/post/voices).
    """

    start_time: typing.Optional[str] = pydantic.Field(default=None)
    """
    The time you want the call to start. If you don't specify a time (or the time is in the past), the call will send immediately.
    
    Set your time in the format `YYYY-MM-DD HH:MM:SS -HH:MM` (ex. `2021-01-01 12:00:00 -05:00`).
    
    The timezone is optional, and defaults to `UTC` if not specified.
    
    Note - Scheduled calls can be cancelled with the `POST /v1/calls/:call_id/stop` endpoint.
    """

    webhook: typing.Optional[Webhook] = pydantic.Field(default=None)
    """
    The webhook should be a http / https callback url. We will send the call_id and transcript to this URL after the call completes. This can be useful if you want to have real time notifications when calls finish.
    """

    wait_for_greeting: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Should the AI speak first or wait for someone else to talk?
    
    Creates more realistic conversations when answered with “Hello?”, “This is {name} speaking.” and so on.
    
        - When `false`, The AI starts speaking shortly after the call is answered.
    
        - When `true`, The AI will wait for the answerer to speak.
    """

    first_sentence: typing.Optional[str] = pydantic.Field(default=None)
    """
    A phrase that your call will start with instead of a generating one on the fly. This works both with and without `wait_for_greeting`. Can be more than one sentence, but must be less than 200 characters.
    """

    record: typing.Optional[bool] = pydantic.Field(default=None)
    """
    To record your phone call, set `record` to true. When your call completes, you can access through the `recording_url` field in the call details or your webhook.
    """

    voice_settings: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Should the AI speak first or wait for someone else to talk? Creates more realistic conversations when answered with “Hello?”, “This is {name} speaking.” and so on. When `false` - The AI starts speaking shortly after the call is answered. When `true` - The AI will wait for the answerer to speak.
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    Select a supported language of your choice. Optimizes every part of our API for that language - transcription, speech, and other inner workings.
    
    Supported Languages and their codes -
    
    - English - `eng`
    - Spanish - `esp`
    - French - `fre`
    - Polish - `pol`
    """

    max_duration: typing.Optional[int] = pydantic.Field(default=None)
    """
    Set the longest you want the call to possibly go in minutes. After the max_duration minutes have passed, the call will automatically end. Example Values - `20, 2`
    """

    voicemail_action: typing.Optional[VoicemailAction] = pydantic.Field(default=None)
    """
    Voicemail action tells the AI what to do when encountering a voicemail. This has 96% accuracy. There is no such thing as a perfect VM detection, but this gets close.
    
    The default value is hangup to save money and keep most users in compliance.
    
    Note - **Leaving voicemails is strongly discouraged.**
    """

    amd: typing.Optional[bool] = pydantic.Field(default=None)
    """
    AMD mode helps our AI navigate phone trees and IVR systems. If you know your call will hit an automated system you should switch it on.
    
    NOTE - AMD mode causes increased delay for the first response, even if answered by a human. Highly recommended to set to `false` in the majority of cases.
    """

    request_data: typing.Optional[typing.Dict[str, typing.Any]] = pydantic.Field(default=None)
    """
    When you want your AI to “know” a specific fact - like the caller's name or other relevant context.
    
    The AI agent will be aware of both the key names as well as their corresponding values.
    """

    dynamic_data: typing.Optional[typing.List[DynamicData]] = pydantic.Field(default=None)
    """
    Make dynamic requests to external APIs and use the data in your AI's responses.
    """

    interruption_threshold: InterruptionThreshold = pydantic.Field()
    """
    When you increase the interruption latency, you force the AI phone agent to listen longer before responding. In practice, increasing the threshold results in less interruptions and more latency.
    
    Try setting the threshold to `500` milliseconds. You'll encounter higher latency, but you'll be interrupted much less frequently.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..commons.errors.server_error import ServerError
from ..commons.errors.unauthorized_error import UnauthorizedError
from ..commons.types.error_body import ErrorBody
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from .types.analyze_call import AnalyzeCall
from .types.analyze_call_response import AnalyzeCallResponse
from .types.audio_recording_response import AudioRecordingResponse
from .types.call_details_response import CallDetailsResponse
from .types.call_id_param import CallIdParam
from .types.list_calls_response import ListCallsResponse
from .types.stop_call_response import StopCallResponse

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CallsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def analyze(
        self, call_id: CallIdParam, *, request: AnalyzeCall, request_options: typing.Optional[RequestOptions] = None
    ) -> AnalyzeCallResponse:
        """
        Analyzes a call of calls based using questions and goals.

        Parameters:
            - call_id: CallIdParam.

            - request: AnalyzeCall.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/calls/{jsonable_encoder(call_id)}/analyze"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(AnalyzeCallResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def stop(
        self, call_id: CallIdParam, *, request_options: typing.Optional[RequestOptions] = None
    ) -> StopCallResponse:
        """
        End an active phone call by call_id.

        Parameters:
            - call_id: CallIdParam.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/calls/{jsonable_encoder(call_id)}/stop"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(StopCallResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list(
        self,
        *,
        from_number: typing.Optional[str] = None,
        to_number: typing.Optional[str] = None,
        from_: typing.Optional[int] = None,
        to: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        ascending: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListCallsResponse:
        """
        Returns a set of metadata for each call dispatched by your account.

        Parameters:
            - from_number: typing.Optional[str]. Filter calls by the number they were dispatched from.

                                                 The number that initiated the call - the user's phone number for inbound calls, or the number your AI Agent called from for outbound calls.
            - to_number: typing.Optional[str]. Filter calls by the number they were dispatched to.

                                               The number that answered the call - the user's phone number for outbound calls, or your AI Agent's number for inbound calls.
            - from_: typing.Optional[int]. The starting index (inclusive) for the range of calls to retrieve.

            - to: typing.Optional[int]. The ending index for the range of calls to retrieve.

            - limit: typing.Optional[int]. The maximum number of calls to return in the response.

            - ascending: typing.Optional[bool]. Whether to sort the calls in ascending order of their creation time.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/calls"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "from_number": from_number,
                        "to_number": to_number,
                        "from": from_,
                        "to": to,
                        "limit": limit,
                        "ascending": ascending,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListCallsResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def details(
        self, call_id: CallIdParam, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CallDetailsResponse:
        """
        Retrieve detailed information, metadata and transcripts for a call.

        Parameters:
            - call_id: CallIdParam.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/calls/{jsonable_encoder(call_id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CallDetailsResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def recording(
        self, call_id: CallIdParam, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AudioRecordingResponse:
        """
        Retrieve your call's audio recording.

        Parameters:
            - call_id: CallIdParam.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/calls/{jsonable_encoder(call_id)}/recording"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(AudioRecordingResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncCallsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def analyze(
        self, call_id: CallIdParam, *, request: AnalyzeCall, request_options: typing.Optional[RequestOptions] = None
    ) -> AnalyzeCallResponse:
        """
        Analyzes a call of calls based using questions and goals.

        Parameters:
            - call_id: CallIdParam.

            - request: AnalyzeCall.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/calls/{jsonable_encoder(call_id)}/analyze"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(AnalyzeCallResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def stop(
        self, call_id: CallIdParam, *, request_options: typing.Optional[RequestOptions] = None
    ) -> StopCallResponse:
        """
        End an active phone call by call_id.

        Parameters:
            - call_id: CallIdParam.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/calls/{jsonable_encoder(call_id)}/stop"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))
            if request_options is not None
            else None,
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(StopCallResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list(
        self,
        *,
        from_number: typing.Optional[str] = None,
        to_number: typing.Optional[str] = None,
        from_: typing.Optional[int] = None,
        to: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        ascending: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListCallsResponse:
        """
        Returns a set of metadata for each call dispatched by your account.

        Parameters:
            - from_number: typing.Optional[str]. Filter calls by the number they were dispatched from.

                                                 The number that initiated the call - the user's phone number for inbound calls, or the number your AI Agent called from for outbound calls.
            - to_number: typing.Optional[str]. Filter calls by the number they were dispatched to.

                                               The number that answered the call - the user's phone number for outbound calls, or your AI Agent's number for inbound calls.
            - from_: typing.Optional[int]. The starting index (inclusive) for the range of calls to retrieve.

            - to: typing.Optional[int]. The ending index for the range of calls to retrieve.

            - limit: typing.Optional[int]. The maximum number of calls to return in the response.

            - ascending: typing.Optional[bool]. Whether to sort the calls in ascending order of their creation time.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "v1/calls"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "from_number": from_number,
                        "to_number": to_number,
                        "from": from_,
                        "to": to,
                        "limit": limit,
                        "ascending": ascending,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ListCallsResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def details(
        self, call_id: CallIdParam, *, request_options: typing.Optional[RequestOptions] = None
    ) -> CallDetailsResponse:
        """
        Retrieve detailed information, metadata and transcripts for a call.

        Parameters:
            - call_id: CallIdParam.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"v1/calls/{jsonable_encoder(call_id)}"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CallDetailsResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def recording(
        self, call_id: CallIdParam, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AudioRecordingResponse:
        """
        Retrieve your call's audio recording.

        Parameters:
            - call_id: CallIdParam.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"v1/calls/{jsonable_encoder(call_id)}/recording"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else 60,
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(AudioRecordingResponse, _response.json())  # type: ignore
        if _response.status_code == 500:
            raise ServerError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        if _response.status_code == 401:
            raise UnauthorizedError(pydantic.parse_obj_as(ErrorBody, _response.json()))  # type: ignore
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class RetrievedVoice(pydantic.BaseModel):
    voice_preset_id: str = pydantic.Field()
    """
    The unique identifier for the voice preset.
    """

    voice_name: str = pydantic.Field()
    """
    Public voice name.
    """

    description: str = pydantic.Field()
    """
    The description of the voice preset.
    """

    reduce_latency: bool = pydantic.Field()
    """
    Whether or not the voice preset is optimized for low latency.
    """

    interruption_threshold: typing.Optional[int] = pydantic.Field(default=None)
    """
    The threshold for the voice preset to be interrupted by a new message.
    """

    language: str = pydantic.Field()
    """
    The language code of the voice preset.
    """

    voice_settings: typing.Any = pydantic.Field()
    """
    The voice settings of the voice preset.
    """

    editable: bool = pydantic.Field()
    """
    Whether or not your account can edit the preset.
    """

    public: bool = pydantic.Field()
    """
    Whether or not the preset is publicly available.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    The date and time the preset was originally created.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class VoiceSettings(pydantic.BaseModel):
    stability: typing.Optional[float] = pydantic.Field(default=None)
    """
    Note: This is an experimental parameter and may behave unexpectedly.
    
    Adjust the predictability and consistency of the AI agent's voice. Lower values allow larger deviations from the baseline voice, whether default or cloned. Setting this too high however can cause a monotone voice.
    
    Accepts decimal values between `0` and `1` (inclusive).
    """

    similarity: typing.Optional[float] = pydantic.Field(default=None)
    """
    Note: This is an experimental parameter and may behave unexpectedly.
    
    Higher values will make speech differences between the selected voice and others more prominent. Extremely high values can cause voice distortion.
    
    Use lower values to lower the distinctiveness of the voice or eliminate unwanted audio static spikes.
    
    Accepts decimal values between `0` and `1` (inclusive).
    """

    speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    Note: This is an experimental parameter and may behave unexpectedly.
    
    Note #2: Setting `reduce_latency` to false will cause this parameter to be ignored.
    
    How fast your agent talks! This parameter is simply a speech-speed multiplier, and works with fractional values such as `0.5` or large ones like `2`.
    
    Accepts decimal values between `0.1` and `5` (inclusive).
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...commons.types.phone_number import PhoneNumber
from ...commons.types.voice_id import VoiceId
from ...commons.types.webhook import Webhook
from ...core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class InboundNumberDetailsResponse(pydantic.BaseModel):
    created_at: str = pydantic.Field()
    """
    The timestamp when the inbound number was configured.
    """

    phone_number: PhoneNumber = pydantic.Field()
    """
    The specific inbound phone number.
    """

    prompt: str = pydantic.Field()
    """
    The prompt your agent is using.
    """

    webhook: Webhook
    voice_id: VoiceId
    dynamic_data: typing.Optional[str] = pydantic.Field(default=None)
    """
    Any dynamic data associated with the inbound number, if applicable.
    """

    max_duration: int = pydantic.Field()
    """
    The maximum duration of a call to the inbound number, in minutes.
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..commons.types.call_id import CallId
from ..commons.types.status import Status
from ..core.datetime_utils import serialize_datetime

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class SendCallResponse(pydantic.BaseModel):
    """
    from blandai import SendCallResponse

    SendCallResponse(
        status="success",
        message="Call successfully queued.",
        call_id="73c809a9-db35-4edb-8fa6-95a6a942daf8",
    )
    """

    status: Status = pydantic.Field()
    """
    Can be `success` or `error`.
    """

    call_id: CallId = pydantic.Field()
    """
    A unique identifier for the call (present only if status is `success`).
    """

    batch_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier for the batcarrayh of calls (present only if status is `success`).
    
    If no batch is specified, this value will be `null`.
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    A message describing the status of the call (present only if status is `success`).
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}
